// SPHGEN_CPP Sphere.hpp
//
// Andrew Magis <amagis@cise.ufl.edu>
// Last change: Time-stamp: <26-Jul-2006 10:25:00 amagis>
//
// Version 1.1
// This software is open source, to be used in conjunction with the UCSF DOCK
// package.  If you have questions, comments, or suggestions, please contact
// me at the email address above.
//

#ifndef SPHERE_HPP
#define SPHERE_HPP

#include <stdio.h>
#include <string.h>
#include <math.h>

struct Sphere;
struct sphereThreadArgs;
class Atom;
class SurfacePoint;
class Vector3d; 

class Vector3d
{
	friend class SurfacePoint;
	friend class Atom;
	
public:
	Vector3d();
	Vector3d(double _x, double _y, double _z);
	virtual ~Vector3d();

	//Returns the class name.  Good for debugging.
	virtual const char * isA() const { return "SurfacePoint"; };
	
	Vector3d operator +(const Vector3d &v);
	Vector3d operator -(const Vector3d &v);
	Vector3d& operator =(const Vector3d &v);
	double operator *(const Vector3d &v);
	void Print(FILE *out);

	double x, y, z;
	double c[3];
	
protected:

	//x, y, and z WERE protected.

};


class SurfacePoint
{
public:
	SurfacePoint();
	SurfacePoint(char *_tag, double _x, double _y, double _z,
				 double _nx, double _ny, double _nz, int _atom_number);
	virtual ~SurfacePoint();
	
	//Returns the class name.  Good for debugging.
	virtual const char * isA() const { return "SurfacePoint"; };
	int CompareSurfacePoints(Atom *a);
	Sphere* WithinLimits(double dotlim, double radmin);
	double GetDotProduct() { return dotproduct; };
	Vector3d* GetPosition() { return position; };
	SurfacePoint* GetNearest() { return nearest; };
	int GetAtomNumber() { return atom_number; };
	void Print(FILE *out);

protected:
	char tag[3];
	int atom_number;
	Vector3d *position;
	Vector3d *normal;
	double cs[3];

	//Sphere generation variables
	double dotproduct;
	SurfacePoint *nearest;
	double slope[2];
	double beta[2];
	double rmin, rtmin, rtmina, rtminb, rtminc;
	int izero;
	
};

class Atom
{
public:
	Atom();
	Atom(double _x, double _y, double _z, char *_residue_name,
		 char *_residue, char *_atom_name, int _num_surfacepoints,
		 int _atom_number);
	virtual ~Atom();
	bool AddSurfacePoint(char *_tag, double _x, double _y, double _z,
						 double _nx, double _ny, double _nz);

	//Returns the class name.  Good for debugging.
	virtual const char * isA() const { return "Atom"; };
	int GetNumSurfacePoints() { return num_surfacepoints; };
	SurfacePoint* GetSurfacePoint(int i) { return points[i]; };
	int CompareAtoms(Atom *a);
	int GenerateSpheres(double dotlim, double radmin, double radmax);
	int GetNumSpheres() { return num_spheres; };
	Sphere* GetSphere(int i) { return spheres[i]; };
	Sphere* GetLargestSphere() { return largest; };
	void Print(FILE *out);

protected:
	char residue_name[3], atom_name[3], residue[4];
	int num_surfacepoints, atom_number, counter, num_spheres;
	SurfacePoint **points;
	Sphere **spheres;
	Vector3d *position;
	Sphere *largest;
	double rtmax;

};

//Structure to store sphere information
struct Sphere {
	SurfacePoint *i;
	SurfacePoint *ipair;
	int this_atom;
	int pair_atom;
	double rtmin;
	Vector3d *cs;
	Sphere **pairs;
	int num_pairs;
	bool is_white;
	Sphere *next;
	Sphere *prev;
	int cluster_size;
};

//Structure for passing arguements to the sphere-wrapping thread
struct sphereThreadArgs 
{
	double dotlim; 
	double radmin; 
	double radmax; 
	char surftp; 
	int startIndex; 
	int stopIndex;
	int threadID;
	
};
#endif // SPHERE_HPP
